@@ -2,9 +2,8 @@
  *  Copyright (C) 2002 - 2005 Tomasz Kojm <tkojm@clamav.net>
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU General Public License version 2 as
+ *  published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -56,7 +55,7 @@ int tcpserver(const struct cfgstruct *copt)
 {
 	struct sockaddr_in server;
 	int sockfd, backlog;
-	struct cfgstruct *taddr;
+	const struct cfgstruct *taddr;
 	struct hostent he;
 	char *estr, buf[1024];
 	int true = 1;
