@@ -59,9 +59,6 @@ static struct option clamscan_longopt[] = {
     {"max-recursion", 1, 0, 0},
     {"max-dir-recursion", 1, 0, 0},
     {"max-mail-recursion", 1, 0, 0},
-#ifdef HAVE_NCORE
-    {"ncore", 0, 0, 0},
-#endif
     {"detect-pua", 0, 0, 0},
     {"disable-archive", 0, 0, 0},
     {"no-archive", 0, 0, 0},
@@ -93,6 +90,7 @@ static struct option clamscan_longopt[] = {
 
     /* developers only */
     {"dev-ac-only", 0, 0, 0},
+    {"dev-ac-depth", 1, 0, 0},
 
     {0, 0, 0, 0}
 };
