@@ -630,7 +630,7 @@ static void move_infected(const char *filename, const struct optstruct *opt)
 	}
     }
 
-    logg("%s: %s to '%s'\n", filename, (moveflag) ? "moved" : "copied", movefilename);
+    logg("~%s: %s to '%s'\n", filename, (moveflag) ? "moved" : "copied", movefilename);
 
     free(movefilename);
 }
@@ -649,7 +649,7 @@ static int checkfile(const char *filename, const struct cl_engine *engine, const
     }
 
     if((ret = cl_scandesc(fd, &virname, &info.blocks, engine, limits, options)) == CL_VIRUS) {
-	logg("%s: %s FOUND\n", filename, virname);
+	logg("~%s: %s FOUND\n", filename, virname);
 	info.ifiles++;
 
 	if(bell)
@@ -657,10 +657,10 @@ static int checkfile(const char *filename, const struct cl_engine *engine, const
 
     } else if(ret == CL_CLEAN) {
 	if(!printinfected && printclean)
-	    mprintf("%s: OK\n", filename);
+	    mprintf("~%s: OK\n", filename);
     } else
 	if(!printinfected)
-	    logg("%s: %s\n", filename, cl_strerror(ret));
+	    logg("~%s: %s\n", filename, cl_strerror(ret));
 
     close(fd);
     return ret;
@@ -822,7 +822,7 @@ static int scancompressed(const char *filename, struct cl_engine *engine, const
 			logg("^%s: Can't remove\n", filename);
 			info.notremoved++;
 		    } else {
-			logg("%s: Removed\n", filename);
+			logg("~%s: Removed\n", filename);
 		    }
 		} else if (opt_check(opt, "move") || opt_check(opt, "copy"))
 		    move_infected(filename, opt);
@@ -839,14 +839,14 @@ static int scancompressed(const char *filename, struct cl_engine *engine, const
 			logg("^%s: Can't remove\n", filename);
 			info.notremoved++;
 		    } else {
-			logg("%s: Removed\n", filename);
+			logg("~%s: Removed\n", filename);
 		    }
 		} else if (opt_check(opt, "move") || opt_check(opt, "copy"))
 		    move_infected(filename, opt);
 	    }
 	    return ret;
 	case 1:
-	    logg("%s: Infected.Archive FOUND\n", filename);
+	    logg("~%s: Infected.Archive FOUND\n", filename);
 
 	    if(bell)
 		fprintf(stderr, "\007");
@@ -856,7 +856,7 @@ static int scancompressed(const char *filename, struct cl_engine *engine, const
 		    logg("^%s: Can't remove\n", filename);
 		    info.notremoved++;
 		} else {
-		    logg("%s: Removed\n", filename);
+		    logg("~%s: Removed\n", filename);
 		}
 	    } else if (opt_check(opt, "move") || opt_check(opt, "copy"))
 		move_infected(filename, opt);
@@ -937,7 +937,7 @@ static int scandenied(const char *filename, struct cl_engine *engine, const stru
 		logg("^%s: Can't remove\n", filename);
 		info.notremoved++;
 	    } else {
-	        logg("%s: Removed\n", filename);
+	        logg("~%s: Removed\n", filename);
 	    }
 	} else if (opt_check(opt, "move") || opt_check(opt, "copy"))
 	    move_infected(filename, opt);
@@ -965,7 +965,7 @@ int scanfile(const char *filename, struct cl_engine *engine, const struct passwd
 	if(stat(filename, &sb) != -1)
 	    if(sb.st_dev == procdev) {
 		if(!printinfected)
-		    logg("%s: Excluded (/proc)\n", filename);
+		    logg("~%s: Excluded (/proc)\n", filename);
 		return 0;
 	    }
 #endif    
@@ -975,7 +975,7 @@ int scanfile(const char *filename, struct cl_engine *engine, const struct passwd
 	while(argument) {
 	    if(match_regex(filename, argument) == 1) {
 		if(!printinfected)
-		    logg("%s: Excluded\n", filename);
+		    logg("~%s: Excluded\n", filename);
 		return 0;
 	    }
 	    argument = opt_nextarg(&optnode, "exclude");
@@ -995,14 +995,14 @@ int scanfile(const char *filename, struct cl_engine *engine, const struct passwd
 
 	if(!included) {
 	    if(!printinfected)
-		logg("%s: Excluded\n", filename);
+		logg("~%s: Excluded\n", filename);
 	    return 0;
 	}
     }
 
     if(fileinfo(filename, 1) == 0) {
 	if(!printinfected)
-	    logg("%s: Empty file\n", filename);
+	    logg("~%s: Empty file\n", filename);
 	return 0;
     }
 
@@ -1010,7 +1010,7 @@ int scanfile(const char *filename, struct cl_engine *engine, const struct passwd
     if(geteuid())
 	if(checkaccess(filename, NULL, R_OK) != 1) {
 	    if(!printinfected)
-		logg("%s: Access denied\n", filename);
+		logg("~%s: Access denied\n", filename);
 	    return 0;
 	}
 #endif
@@ -1032,7 +1032,7 @@ int scanfile(const char *filename, struct cl_engine *engine, const struct passwd
 		    logg("^%s: Can't remove\n", filename);
 		    info.notremoved++;
 		} else {
-		    logg("%s: Removed\n", filename);
+		    logg("~%s: Removed\n", filename);
 		}
 	    } else if (opt_check(opt, "move") || opt_check(opt, "copy"))
 		move_infected(filename, opt);
@@ -1096,7 +1096,7 @@ int scanfile(const char *filename, struct cl_engine *engine, const struct passwd
 		logg("^%s: Can't remove\n", filename);
 		info.notremoved++;
 	    } else {
-		logg("%s: Removed\n", filename);
+		logg("~%s: Removed\n", filename);
 	    }
 	} else if (opt_check(opt, "move") || opt_check(opt, "copy"))
             move_infected(filename, opt);
