@@ -247,12 +247,6 @@ int main(int argc, char **argv)
 	return 0;
     }
 
-    if(opt_check(opt, "version")) {
-	print_version();
-	opt_free(opt);
-	return 0;
-    }
-
     /* parse the config file */
     if((cfgfile = opt_arg(opt, "config-file"))) {
 	copt = getcfg(cfgfile, 1);
@@ -268,6 +262,18 @@ int main(int argc, char **argv)
 	return 56;
     }
 
+    if(opt_check(opt, "datadir"))
+	newdir = opt_arg(opt, "datadir");
+    else
+	newdir = cfgopt(copt, "DatabaseDirectory")->strarg;
+
+    if(opt_check(opt, "version")) {
+	print_version(newdir);
+	opt_free(opt);
+	freecfg(copt);
+	return 0;
+    }
+
 #ifdef C_WINDOWS
     if(!pthread_win32_process_attach_np()) {
 	mprintf("!Can't start the win32 pthreads layer\n");
@@ -406,11 +412,6 @@ int main(int argc, char **argv)
 #endif
 
     /* change the current working directory */
-    if(opt_check(opt, "datadir"))
-	newdir = opt_arg(opt, "datadir");
-    else
-	newdir = cfgopt(copt, "DatabaseDirectory")->strarg;
-
     if(chdir(newdir)) {
 	logg("Can't change dir to %s\n", newdir);
 	opt_free(opt);
@@ -480,8 +481,13 @@ int main(int argc, char **argv)
 	bigsleep = 24 * 3600 / checks;
 
 	if(!cfgopt(copt, "Foreground")->enabled) {
+	    if(daemonize() == -1) {
+		logg("!daemonize() failed\n");
+		opt_free(opt);
+		freecfg(copt);
+		return 70; /* FIXME */
+	    }
             foreground = 0;
-	    daemonize();
 	    mprintf_disabled = 1;
         }
 
