@@ -1,10 +1,9 @@
 /*
- *  Copyright (C) 2003 Tomasz Kojm <zolw@konarski.edu.pl>
+ *  Copyright (C) 2003 Tomasz Kojm <tkojm@clamav.net>
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU General Public License version 2 as
+ *  published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,7 +12,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ *  MA 02110-1301, USA.
  */
 
 #ifndef __CVD_H
@@ -22,7 +22,7 @@
 #include <stdio.h>
 #include "clamav.h"
 
-int cli_cvdload(FILE *fd, struct cl_node **root, unsigned int *signo, short warn);
+int cli_cvdload(FILE *fs, struct cl_engine **engine, unsigned int *signo, short warn, unsigned int options);
 int cli_untgz(int fd, const char *destdir);
 
 #endif
