@@ -4,9 +4,8 @@
  *  Copyright (C) 2006-2007 Trk Edvin <edwin@clamav.net>
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU General Public License version 2 as 
+ *  published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,7 +19,6 @@
  *
  */
 #include <clamav-config.h>
-#ifdef  CL_EXPERIMENTAL
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -413,4 +411,3 @@ int hashtab_load(FILE* in, struct hashtable *s)
 	return CL_SUCCESS;
 }
 
-#endif
