@@ -37,8 +37,8 @@ typedef struct m_area_tag {
 
 
 unsigned char *cli_readline(FILE *stream, m_area_t *m_area, unsigned int max_len);
-int html_normalise_mem(unsigned char *in_buff, off_t in_size, const char *dirname, tag_arguments_t *hrefs);
-int html_normalise_fd(int fd, const char *dirname, tag_arguments_t *hrefs);
+int html_normalise_mem(unsigned char *in_buff, off_t in_size, const char *dirname, tag_arguments_t *hrefs,const struct cli_dconf* dconf);
+int html_normalise_fd(int fd, const char *dirname, tag_arguments_t *hrefs,const struct cli_dconf* dconf);
 void html_tag_arg_free(tag_arguments_t *tags);
 int html_screnc_decode(int fd, const char *dirname);
  
