@@ -24,11 +24,12 @@
 #include "cltypes.h"
 
 struct cli_bm_patt {
-    unsigned char *pattern;
-    uint32_t length;
+    unsigned char *pattern, *prefix;
+    uint16_t length, prefix_length;
     char *virname, *offset;
     uint8_t target;
     struct cli_bm_patt *next;
+    uint16_t cnt;
 };
 
 int cli_bm_addpatt(struct cli_matcher *root, struct cli_bm_patt *pattern);
