@@ -42,14 +42,8 @@
 #include "str.h"
 #include "cltypes.h"
 
-#ifdef HAVE_NCORE
-#include "matcher-ncore.h"
-#endif
-
 static cli_file_t targettab[CL_TARGET_TABLE_SIZE] = { 0, CL_TYPE_MSEXE, CL_TYPE_MSOLE2, CL_TYPE_HTML, CL_TYPE_MAIL, CL_TYPE_GRAPHICS, CL_TYPE_ELF };
 
-extern short cli_debug_flag;
-
 int cli_scanbuff(const unsigned char *buffer, uint32_t length, const char **virname, const struct cl_engine *engine, cli_file_t ftype)
 {
 	int ret = CL_CLEAN;
@@ -63,11 +57,6 @@ int cli_scanbuff(const unsigned char *buffer, uint32_t length, const char **virn
 	return CL_ENULLARG;
     }
 
-#ifdef HAVE_NCORE
-    if(engine->ncore)
-	return cli_ncore_scanbuff(buffer, length, virname, engine, ftype, targettab);
-#endif
-
     groot = engine->root[0]; /* generic signatures */
 
     if(ftype) {
@@ -274,11 +263,11 @@ int cli_validatesig(cli_file_t ftype, const char *offstr, off_t fileoff, struct
 
 	if(maxshift) {
 	    if((fileoff < offset) || (fileoff > offset + (off_t) maxshift)) {
-		cli_dbgmsg("Signature offset: %lu, expected: [%lu..%lu] (%s)\n", fileoff, offset, offset + maxshift, virname);
+		/* cli_dbgmsg("Signature offset: %lu, expected: [%lu..%lu] (%s)\n", fileoff, offset, offset + maxshift, virname); */
 		return 0;
 	    }
 	} else if(fileoff != offset) {
-	    cli_dbgmsg("Signature offset: %lu, expected: %lu (%s)\n", fileoff, offset, virname);
+	    /* cli_dbgmsg("Signature offset: %lu, expected: %lu (%s)\n", fileoff, offset, virname); */
 	    return 0;
 	}
     }
@@ -303,16 +292,6 @@ int cli_scandesc(int desc, cli_ctx *ctx, uint8_t otfrec, cli_file_t ftype, uint8
 	return CL_ENULLARG;
     }
 
-#ifdef HAVE_NCORE
-    if(ctx->engine->ncore) {
-	    int cont;
-
-	ret = cli_ncore_scandesc(desc, ctx, ftype, &cont, targettab, &md5ctx);
-	if(!cont)
-	    return ret;
-    }
-#endif
-
     if(!ftonly)
 	groot = ctx->engine->root[0]; /* generic signatures */
 
