@@ -28,7 +28,7 @@ int init_domainlist(struct cl_engine* engine);
 void domainlist_done(struct cl_engine* engine);
 void domainlist_cleanup(const struct cl_engine* engine);
 int is_domainlist_ok(const struct cl_engine* engine);
-int domainlist_match(const struct cl_engine* engine,const char* real_url,const char* display_url,int hostOnly,unsigned short* flags);
+int domainlist_match(const struct cl_engine* engine,char* real_url,const char* display_url,const struct pre_fixup_info* pre_fixup, int hostOnly,unsigned short* flags);
 
 #endif
 
