@@ -51,8 +51,6 @@
 #define EC32(x) le32_to_host(x) /* Convert little endian to host */
 #define EC16(x) le16_to_host(x) /* Convert little endian to host */
 
-extern short cli_leavetemps_flag;
-
 static const char *langcodes[] = {
     "",   "EN", "FR", "GE", "SP", "IT", "SW", "DA", "NO", "FI", "AM",
     "SF", "SG", "PO", "TU", "IC", "RU", "HU", "DU", "BL", "AU", "BG",
@@ -79,7 +77,7 @@ static char *sis_utf16_decode(const char *str, uint32_t length)
 
 
     if(!length || length % 2) {
-	cli_warnmsg("SIS: sis_utf16_decode: Broken filename (length == %d)\n", length);
+	cli_dbgmsg("SIS: sis_utf16_decode: Broken filename (length == %d)\n", length);
 	return NULL;
     }
 
@@ -296,7 +294,7 @@ static int sis_extract_simple(int fd, char *mfile, uint32_t length, uint32_t off
 	    } 
 
 	    if(uncompress((Bytef *) buff, &osize , (Bytef *) mfile + fileoff, csize) != Z_OK) {
-		cli_errmsg("SIS: sis_extract_simple: File decompression failed\n");
+		cli_dbgmsg("SIS: sis_extract_simple: File decompression failed\n");
 		free(buff);
 		free(subdir);
 		free(fname);
@@ -327,6 +325,7 @@ static int sis_extract_simple(int fd, char *mfile, uint32_t length, uint32_t off
 	    free(fname);
 	    if(compressed)
 		free(buff);
+	    close(desc);
 	    return CL_EIO;
 	} else {
 	    if(compressed)
@@ -541,19 +540,19 @@ int cli_scansis(int desc, cli_ctx *ctx)
     cli_dbgmsg("SIS: Number of files: %d\n", nfiles);
     cli_dbgmsg("SIS: Offset of files records: %d\n", EC32(file_hdr.pfiles));
 
-    if(!(dir = cli_gentempdir(NULL))) {
-	cli_errmsg("SIS: Can't generate temporary directory\n");
-	munmap(mfile, length);
-	return CL_ETMPDIR;
-    }
-
     if((frecord = EC32(file_hdr.pfiles)) >= length) {
 	cli_errmsg("SIS: Broken file structure (frecord)\n");
 	munmap(mfile, length);
-	free(dir);
 	return CL_EFORMAT;
     }
 
+    dir = cli_gentemp(NULL);
+    if(!dir || mkdir(dir, 0700) == -1) {
+	cli_errmsg("SIS: Can't create temporary directory %s\n", dir ? dir : "");
+	munmap(mfile, length);
+	return CL_ETMPDIR;
+    }
+
     for(i = 0; i < nfiles; i++) {
 
 	cli_dbgmsg("SIS: -----\n");
