@@ -2,9 +2,8 @@
  *  Copyright (C) 2005 aCaB <acab@clamav.net>
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU General Public License version 2 as
+ *  published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,6 +22,6 @@
 #include "cltypes.h"
 #include "rebuildpe.h"
 
-int unspin(char *, int, struct pe_image_section_hdr *, int, uint32_t, int, cli_ctx *);
+int unspin(char *, int, struct cli_exe_section *, int, uint32_t, int, cli_ctx *);
 
 #endif
