@@ -2,9 +2,8 @@
  *  Copyright (C) 2004 aCaB <acab@clamav.net>
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU General Public License version 2 as
+ *  published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,6 +25,11 @@
 ** 04/06/2k4 - Now we handle 2B, 2D and 2E :D
 ** 28/08/2k4 - PE rebuild for nested packers
 ** 12/12/2k4 - Improved PE rebuild code and added some debug info on failure
+** 23/03/2k7 - New approach for rebuilding:
+               o Get imports via magic
+               o Get imports via leascan
+               o if (!pe) pe=scan4pe();
+	       o if (!pe) forgepe();
 */
 
 /*
@@ -49,6 +53,7 @@
 #include "cltypes.h"
 #include "others.h"
 #include "upx.h"
+#include "str.h"
 
 #define PEALIGN(o,a) (((a))?(((o)/(a))*(a)):(o))
 #define PESALIGN(o,a) (((a))?(((o)/(a)+((o)%(a)!=0))*(a)):(o))
@@ -68,72 +73,138 @@
 \x65\x72\x20\x2D\x20\x68\x74\x74\x70\x3A\x2F\x2F\x77\x77\x77\x2E\
 \x63\x6C\x61\x6D\x61\x76\x2E\x6E\x65\x74\x0D\x0A\x24\x00\x00\x00\
 "
+#define FAKEPE "\
+\x50\x45\x00\x00\x4C\x01\x01\x00\x43\x4C\x41\x4D\x00\x00\x00\x00\
+\x00\x00\x00\x00\xE0\x00\x83\x8F\x0B\x01\x00\x00\x00\x10\x00\x00\
+\x00\x10\x00\x00\x00\x00\x00\x00\x00\x10\x00\x00\x00\x10\x00\x00\
+\x00\x10\x00\x00\x00\x00\x40\x00\x00\x10\x00\x00\x00\x02\x00\x00\
+\x01\x00\x00\x00\x00\x00\x00\x00\x03\x00\x0A\x00\x00\x00\x00\x00\
+\xFF\xFF\xFF\xFF\x00\x02\x00\x00\x00\x00\x00\x00\x02\x00\x00\x00\
+\x00\x00\x10\x00\x00\x10\x00\x00\x00\x00\x10\x00\x00\x10\x00\x00\
+\x00\x00\x00\x00\x10\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\
+\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\
+\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\
+\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\
+\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\
+\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\
+\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\
+\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\
+\x00\x00\x00\x00\x00\x00\x00\x00\x2e\x63\x6c\x61\x6d\x30\x31\x00\
+\xFF\xFF\xFF\xFF\x00\x10\x00\x00\xFF\xFF\xFF\xFF\x00\x02\x00\x00\
+\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xff\xff\xff\xff\
+"
+
+static char *checkpe(char *dst, uint32_t dsize, char *pehdr, uint32_t *valign, unsigned int *sectcnt)
+{
+  char *sections;
+  if (!CLI_ISCONTAINED(dst, dsize,  pehdr, 0xf8)) return NULL;
+
+  if (cli_readint32(pehdr) != 0x4550 ) return NULL;
+  
+  if (!(*valign=cli_readint32(pehdr+0x38))) return NULL;
+  
+  sections = pehdr+0xf8;
+  if (!(*sectcnt = (unsigned char)pehdr[6] + (unsigned char)pehdr[7]*256)) return NULL;
+  
+  if (!CLI_ISCONTAINED(dst, dsize, sections, *sectcnt*0x28)) return NULL;
+
+  return sections;
+}
 
 /* PE from UPX */
 
-static int pefromupx (char *src, char *dst, uint32_t *dsize, uint32_t ep, uint32_t upx0, uint32_t upx1, uint32_t magic)
+static int pefromupx (char *src, uint32_t ssize, char *dst, uint32_t *dsize, uint32_t ep, uint32_t upx0, uint32_t upx1, uint32_t *magic, uint32_t dend)
 {
-  char *imports, *sections, *pehdr, *newbuf;
-  int sectcnt, upd=1;
-  uint32_t realstuffsz, valign;
+  char *imports, *sections, *pehdr=NULL, *newbuf;
+  unsigned int sectcnt=0, upd=1;
+  uint32_t realstuffsz=0, valign=0;
   uint32_t foffset=0xd0+0xf8;
 
   if((dst == NULL) || (src == NULL))
     return 0;
 
-  imports = dst + cli_readint32(src + ep - upx1 + magic);
-
-  realstuffsz = imports-dst;
-  
-  if (realstuffsz >= *dsize ) {
-    cli_dbgmsg("UPX: wrong realstuff size - giving up rebuild\n");
-    return 0;
+  while ((valign=magic[sectcnt++])) {
+    if ( ep - upx1 + valign <= ssize-5  &&    /* Wondering how we got so far?! */
+	 src[ep - upx1 + valign - 2] == '\x8d' && /* lea edi, ...                  */
+	 src[ep - upx1 + valign - 1] == '\xbe' )  /* ... [esi + offset]          */
+      break;
   }
-  
-  pehdr = imports;
-  while (CLI_ISCONTAINED(dst, *dsize,  pehdr, 8) && cli_readint32(pehdr)) {
-    pehdr+=8;
-    while(CLI_ISCONTAINED(dst, *dsize,  pehdr, 2) && *pehdr) {
-      pehdr++;
-      while (CLI_ISCONTAINED(dst, *dsize,  pehdr, 2) && *pehdr)
-	pehdr++;
-      pehdr++;
+
+  if (!valign && ep - upx1 + 0x80 < ssize-8) {
+    char *pt = &src[ep - upx1 + 0x80];
+    cli_dbgmsg("UPX: bad magic - scanning for imports\n");
+    
+    while ((pt=(char *)cli_memstr(pt, ssize - (pt-src) - 8, "\x8d\xbe", 2))) {
+      if (pt[6] == '\x8b' && pt[7] == '\x07') { /* lea edi, [esi+imports] / mov eax, [edi] */
+	valign=pt-src+2-ep+upx1;
+	break;
+      }
+      pt++;
     }
-    pehdr++;
   }
 
-  pehdr+=4;
-  if (!CLI_ISCONTAINED(dst, *dsize,  pehdr, 0xf8)) {
-    cli_dbgmsg("UPX: sections out of bounds - giving up rebuild\n");
-    return 0;
+  if (valign && CLI_ISCONTAINED(src, ssize, src + ep - upx1 + valign, 4)) {
+    imports = dst + cli_readint32(src + ep - upx1 + valign);
+    
+    realstuffsz = imports-dst;
+    
+    if (realstuffsz >= *dsize ) {
+      cli_dbgmsg("UPX: wrong realstuff size\n");
+      /* fallback and eventually craft */
+    } else {
+      pehdr = imports;
+      while (CLI_ISCONTAINED(dst, *dsize,  pehdr, 8) && cli_readint32(pehdr)) {
+	pehdr+=8;
+	while(CLI_ISCONTAINED(dst, *dsize,  pehdr, 2) && *pehdr) {
+	  pehdr++;
+	  while (CLI_ISCONTAINED(dst, *dsize,  pehdr, 2) && *pehdr)
+	    pehdr++;
+	  pehdr++;
+	}
+	pehdr++;
+      }
+      
+      pehdr+=4;
+      if (!(sections=checkpe(dst, *dsize, pehdr, &valign, &sectcnt))) pehdr=NULL;
+    }
   }
-  
-  if ( cli_readint32(pehdr) != 0x4550 ) {
-    cli_dbgmsg("UPX: No magic for PE - giving up rebuild\n");
-    return 0;
+
+  if (!pehdr && dend>0xf8+0x28) {
+    cli_dbgmsg("UPX: no luck - scanning for PE\n");
+    pehdr = &dst[dend-0xf8-0x28];
+    while (pehdr>dst) {
+      if ((sections=checkpe(dst, *dsize, pehdr, &valign, &sectcnt)))
+	break;
+      pehdr--;
+    }
+    if (!(realstuffsz = pehdr-dst)) pehdr=NULL;
   }
-  
-  if (!(valign=cli_readint32(pehdr+0x38))) {
-    cli_dbgmsg("UPX: Cant align to a NULL bound - giving up rebuild\n");
-    return 0;
-  }
-  
-  sections = pehdr+0xf8;
-  if ( ! (sectcnt = (unsigned char)pehdr[6]+256*(unsigned char)pehdr[7])) {
-    cli_dbgmsg("UPX: No sections? - giving up rebuild\n");
-    return 0;
+
+  if (!pehdr) {
+    uint32_t rebsz = PESALIGN(dend, 0x1000);
+    cli_dbgmsg("UPX: no luck - brutally crafing a reasonable PE\n");
+    if (!(newbuf = (char *)cli_calloc(rebsz+0x200, sizeof(char)))) {
+      cli_dbgmsg("UPX: malloc failed - giving up rebuild\n");
+      return 0;
+    }
+    memcpy(newbuf, HEADERS, 0xd0);
+    memcpy(newbuf+0xd0, FAKEPE, 0x120);
+    memcpy(newbuf+0x200, dst, dend);
+    memcpy(dst, newbuf, dend+0x200);
+    free(newbuf);
+    cli_writeint32(dst+0xd0+0x50, rebsz+0x1000);
+    cli_writeint32(dst+0xd0+0x100, rebsz);
+    cli_writeint32(dst+0xd0+0x108, rebsz);
+    *dsize=rebsz+0x200;
+    cli_dbgmsg("UPX: PE structure added to uncompressed data\n");
+    return 1;
   }
   
   foffset = PESALIGN(foffset+0x28*sectcnt, valign);
   
-  if (!CLI_ISCONTAINED(dst, *dsize, sections, 0x28*sectcnt)) {
-    cli_dbgmsg("UPX: Not enough space for all sects - giving up rebuild\n");
-    return 0;
-  }
-  
   for (upd = 0; upd <sectcnt ; upd++) {
-    uint32_t vsize=PESALIGN(cli_readint32(sections+8), valign);
-    uint32_t urva=PEALIGN(cli_readint32(sections+12), valign);
+    uint32_t vsize=PESALIGN((uint32_t)cli_readint32(sections+8), valign);
+    uint32_t urva=PEALIGN((uint32_t)cli_readint32(sections+12), valign);
     
     /* Within bounds ? */
     if (!CLI_ISCONTAINED(upx0, realstuffsz, urva, vsize)) {
@@ -185,9 +256,9 @@ static int pefromupx (char *src, char *dst, uint32_t *dsize, uint32_t ep, uint32
 
 /* [doubleebx] */
 
-static int doubleebx(char *src, int32_t *myebx, int *scur, int ssize)
+static int doubleebx(char *src, uint32_t *myebx, uint32_t *scur, uint32_t ssize)
 {
-  int32_t oldebx = *myebx;
+  uint32_t oldebx = *myebx;
 
   *myebx*=2;
   if ( !(oldebx & 0x7fffffff)) {
@@ -197,19 +268,20 @@ static int doubleebx(char *src, int32_t *myebx, int *scur, int ssize)
     *myebx = oldebx*2+1;
     *scur+=4;
   }
-  return (oldebx>>31)&1;
+  return (oldebx>>31);
 }
 
 /* [inflate] */
 
 int upx_inflate2b(char *src, uint32_t ssize, char *dst, uint32_t *dsize, uint32_t upx0, uint32_t upx1, uint32_t ep)
 {
-  int32_t backbytes, unp_offset = -1, myebx = 0;
-  int scur=0, dcur=0, i, backsize, oob;
-
+  int32_t backbytes, unp_offset = -1;
+  uint32_t backsize, myebx = 0, scur=0, dcur=0, i, magic[]={0x108,0x110,0xd5,0};
+  int oob;
+  
   while (1) {
     while ((oob = doubleebx(src, &myebx, &scur, ssize)) == 1) {
-      if (scur<0 || scur>=ssize || dcur<0 || dcur>=*dsize)
+      if (scur>=ssize || dcur>=*dsize)
 	return -1;
       dst[dcur++] = src[scur++];
     }
@@ -229,12 +301,11 @@ int upx_inflate2b(char *src, uint32_t ssize, char *dst, uint32_t *dsize, uint32_
         break;
     }
 
-    backsize = 0;
     backbytes-=3;
   
     if ( backbytes >= 0 ) {
 
-      if (scur<0 || scur>=ssize)
+      if (scur>=ssize)
 	return -1;
       backbytes<<=8;
       backbytes+=(unsigned char)(src[scur++]);
@@ -245,9 +316,8 @@ int upx_inflate2b(char *src, uint32_t ssize, char *dst, uint32_t *dsize, uint32_
       unp_offset = backbytes;
     }
 
-    if ( (oob = doubleebx(src, &myebx, &scur, ssize)) == -1)
+    if ( (backsize = (uint32_t)doubleebx(src, &myebx, &scur, ssize)) == 0xffffffff)
       return -1;
-    backsize = oob;
     if ( (oob = doubleebx(src, &myebx, &scur, ssize)) == -1)
       return -1;
     backsize = backsize*2 + oob;
@@ -268,32 +338,25 @@ int upx_inflate2b(char *src, uint32_t ssize, char *dst, uint32_t *dsize, uint32_
 
     backsize++;
 
-    for (i = 0; i < backsize; i++) {
-      if (dcur+i<0 || dcur+i>=*dsize || dcur+unp_offset+i<0 || dcur+unp_offset+i>=*dsize)
-	return -1;
+    if (!CLI_ISCONTAINED(dst, *dsize, dst+dcur+unp_offset, backsize) || !CLI_ISCONTAINED(dst, *dsize, dst+dcur, backsize) || unp_offset >=0)
+      return -1;
+    for (i = 0; i < backsize; i++)
       dst[dcur + i] = dst[dcur + unp_offset + i];
-    }
     dcur+=backsize;
   }
 
-
-  if ( ep - upx1 + 0x108 <= ssize-5  &&    /* Wondering how we got so far?! */
-       src[ep - upx1 + 0x106] == '\x8d' && /* lea edi, ...                  */
-       src[ep - upx1 + 0x107] == '\xbe' )  /* ... [esi + offset]          */
-    return pefromupx (src, dst, dsize, ep, upx0, upx1, 0x108);
-
-  cli_dbgmsg("UPX: bad magic for 2b\n");
-  return 0;
+  return pefromupx (src, ssize, dst, dsize, ep, upx0, upx1, magic, dcur);
 }
 
 int upx_inflate2d(char *src, uint32_t ssize, char *dst, uint32_t *dsize, uint32_t upx0, uint32_t upx1, uint32_t ep)
 {
-  int32_t backbytes, unp_offset = -1, myebx = 0;
-  int scur=0, dcur=0, i, backsize, oob;
+  int32_t backbytes, unp_offset = -1;
+  uint32_t backsize, myebx = 0, scur=0, dcur=0, i, magic[]={0x11c,0x124,0};
+  int oob;
 
   while (1) {
     while ( (oob = doubleebx(src, &myebx, &scur, ssize)) == 1) {
-      if (scur<0 || scur>=ssize || dcur<0 || dcur>=*dsize)
+      if (scur>=ssize || dcur>=*dsize)
 	return -1;
       dst[dcur++] = src[scur++];
     }
@@ -322,7 +385,7 @@ int upx_inflate2d(char *src, uint32_t ssize, char *dst, uint32_t *dsize, uint32_
   
     if ( backbytes >= 0 ) {
 
-      if (scur<0 || scur>=ssize)
+      if (scur>=ssize)
 	return -1;
       backbytes<<=8;
       backbytes+=(unsigned char)(src[scur++]);
@@ -331,11 +394,10 @@ int upx_inflate2d(char *src, uint32_t ssize, char *dst, uint32_t *dsize, uint32_
       if (!backbytes)
 	break;
       backsize = backbytes & 1;
-      backbytes>>=1;
+      CLI_SAR(backbytes,1);
       unp_offset = backbytes;
-    }
-    else {
-      if ( (backsize = doubleebx(src, &myebx, &scur, ssize)) == -1 )
+    } else {
+      if ( (backsize = (uint32_t)doubleebx(src, &myebx, &scur, ssize)) == 0xffffffff )
         return -1;
     }
  
@@ -358,34 +420,27 @@ int upx_inflate2d(char *src, uint32_t ssize, char *dst, uint32_t *dsize, uint32_
       backsize++;
 
     backsize++;
-    for (i = 0; i < backsize; i++) {
-      if (dcur+i<0 || dcur+i>=*dsize || dcur+unp_offset+i<0 || dcur+unp_offset+i>=*dsize)
-	return -1;
+    if (!CLI_ISCONTAINED(dst, *dsize, dst+dcur+unp_offset, backsize) || !CLI_ISCONTAINED(dst, *dsize, dst+dcur, backsize) || unp_offset >=0 )
+      return -1;
+    for (i = 0; i < backsize; i++)
       dst[dcur + i] = dst[dcur + unp_offset + i];
-    }
     dcur+=backsize;
   }
 
-  if ( ep - upx1 + 0x124 <= ssize-5 ) {   /* Wondering how we got so far?! */
-    if ( src[ep - upx1 + 0x11a] == '\x8d' && src[ep - upx1 + 0x11b] == '\xbe' )
-      return pefromupx (src, dst, dsize, ep, upx0, upx1, 0x11c);
-    if ( src[ep - upx1 + 0x122] == '\x8d' && src[ep - upx1 + 0x123] == '\xbe' )
-      return pefromupx (src, dst, dsize, ep, upx0, upx1, 0x124);
-  }
-  cli_dbgmsg("UPX: bad magic for 2d\n");
-  return 0;
+  return pefromupx (src, ssize, dst, dsize, ep, upx0, upx1, magic, dcur);
 }
 
 int upx_inflate2e(char *src, uint32_t ssize, char *dst, uint32_t *dsize, uint32_t upx0, uint32_t upx1, uint32_t ep)
 {
-  int32_t backbytes, unp_offset = -1, myebx = 0;
-  int scur=0, dcur=0, i, backsize, oob;
+  int32_t backbytes, unp_offset = -1;
+  uint32_t backsize, myebx = 0, scur=0, dcur=0, i, magic[]={0x128,0x130,0};
+  int oob;
 
   for(;;) {
     while ( (oob = doubleebx(src, &myebx, &scur, ssize)) ) {
       if (oob == -1)
         return -1;
-      if (scur<0 || scur>=ssize || dcur<0 || dcur>=*dsize)
+      if (scur>=ssize || dcur>=*dsize)
 	return -1;
       dst[dcur++] = src[scur++];
     }
@@ -406,12 +461,11 @@ int upx_inflate2e(char *src, uint32_t ssize, char *dst, uint32_t *dsize, uint32_
       backbytes=backbytes*2+oob;
     }
 
-    backsize = 0;
     backbytes-=3;
   
     if ( backbytes >= 0 ) {
 
-      if (scur<0 || scur>=ssize)
+      if (scur>=ssize)
 	return -1;
       backbytes<<=8;
       backbytes+=(unsigned char)(src[scur++]);
@@ -420,19 +474,17 @@ int upx_inflate2e(char *src, uint32_t ssize, char *dst, uint32_t *dsize, uint32_
       if (!backbytes)
 	break;
       backsize = backbytes & 1; /* Using backsize to carry on the shifted out bit (UPX uses CF) */
-      backbytes>>=1;
+      CLI_SAR(backbytes,1);
       unp_offset = backbytes;
-    }
-    else {
-      if ( (backsize = doubleebx(src, &myebx, &scur, ssize)) == -1 )
+    } else {
+      if ( (backsize = (uint32_t)doubleebx(src, &myebx, &scur, ssize)) == 0xffffffff )
         return -1;
     } /* Using backsize to carry on the doubleebx result (UPX uses CF) */
 
     if (backsize) { /* i.e. IF ( last sar shifted out 1 bit || last doubleebx()==1 ) */
-      if ( (backsize = doubleebx(src, &myebx, &scur, ssize)) == -1 )
+      if ( (backsize = (uint32_t)doubleebx(src, &myebx, &scur, ssize)) == 0xffffffff )
         return -1;
-    }
-    else {
+    } else {
       backsize = 1;
       if ((oob = doubleebx(src, &myebx, &scur, ssize)) == -1)
         return -1;
@@ -440,37 +492,29 @@ int upx_inflate2e(char *src, uint32_t ssize, char *dst, uint32_t *dsize, uint32_
 	if ((oob = doubleebx(src, &myebx, &scur, ssize)) == -1)
 	  return -1;
 	  backsize = 2 + oob;
+	} else {
+	  do {
+	    if ((oob = doubleebx(src, &myebx, &scur, ssize)) == -1)
+	      return -1;
+	    backsize = backsize * 2 + oob;
+	  } while ((oob = doubleebx(src, &myebx, &scur, ssize)) == 0);
+	  if (oob == -1)
+	    return -1;
+	  backsize+=2;
 	}
-      else {
-	do {
-          if ((oob = doubleebx(src, &myebx, &scur, ssize)) == -1)
-          return -1;
-	  backsize = backsize * 2 + oob;
-	} while ((oob = doubleebx(src, &myebx, &scur, ssize)) == 0);
-	if (oob == -1)
-          return -1;
-	backsize+=2;
-      }
     }
  
     if ( (uint32_t)unp_offset < 0xfffffb00 ) 
       backsize++;
 
     backsize+=2;
-    for (i = 0; i < backsize; i++) {
-      if (dcur+i<0 || dcur+i>=*dsize || dcur+unp_offset+i<0 || dcur+unp_offset+i>=*dsize)
-	return -1;
+
+    if (!CLI_ISCONTAINED(dst, *dsize, dst+dcur+unp_offset, backsize) || !CLI_ISCONTAINED(dst, *dsize, dst+dcur, backsize) || unp_offset >=0 )
+      return -1;
+    for (i = 0; i < backsize; i++)
       dst[dcur + i] = dst[dcur + unp_offset + i];
-    }
     dcur+=backsize;
   }
 
-  if ( ep - upx1 + 0x130 <= ssize-5 ) {   /* Wondering how we got so far?! */
-    if ( src[ep - upx1 + 0x126] == '\x8d' && src[ep - upx1 + 0x127] == '\xbe' )
-      return pefromupx (src, dst, dsize, ep, upx0, upx1, 0x128);
-    if ( src[ep - upx1 + 0x12e] == '\x8d' && src[ep - upx1 + 0x12f] == '\xbe' )
-      return pefromupx (src, dst, dsize, ep, upx0, upx1, 0x130);
-  }
-  cli_dbgmsg("UPX: bad magic for 2e\n");
-  return 0;
+  return pefromupx (src, ssize, dst, dsize, ep, upx0, upx1, magic, dcur);
 }
