@@ -377,7 +377,6 @@ int unrar_open(int fd, const char *dirname, unrar_state_t *state)
     state->main_hdr = main_hdr;
     state->metadata_tail = state->metadata = NULL;
     state->file_count = 1;
-    state->offset = offset;
     state->fd = fd;
 
     return UNRAR_OK;
@@ -399,8 +398,8 @@ int unrar_extract_next_prepare(unrar_state_t *state, const char *dirname)
     if(!new_metadata)
 	return UNRAR_EMEM;
 
-    new_metadata->pack_size = state->file_header->high_pack_size * 0x100000000 + state->file_header->pack_size;
-    new_metadata->unpack_size = state->file_header->high_unpack_size * 0x100000000 + state->file_header->unpack_size;
+    new_metadata->pack_size = state->file_header->high_pack_size * 0x100000000ULL + state->file_header->pack_size;
+    new_metadata->unpack_size = state->file_header->high_unpack_size * 0x100000000ULL + state->file_header->unpack_size;
     new_metadata->crc = state->file_header->file_crc;
     new_metadata->method = state->file_header->method;
     new_metadata->filename = strdup(state->file_header->filename);
@@ -457,7 +456,7 @@ int unrar_extract_next(unrar_state_t *state, const char *dirname)
 
 
     if(lseek(state->fd, state->file_header->start_offset+state->file_header->head_size, SEEK_SET) != state->file_header->start_offset+state->file_header->head_size) {
-	unrar_dbgmsg("UNRAR: Seek failed: %ld\n", state->offset+state->file_header->head_size);
+	unrar_dbgmsg("UNRAR: Seek failed: %ld\n", state->file_header->start_offset+state->file_header->head_size);
 	free(state->file_header->filename);
 	free(state->file_header);
 	return UNRAR_ERR;
