@@ -33,11 +33,11 @@
 #include "cfgparser.h"
 
 char *freshdbdir(void);
-void print_version(void);
+void print_version(const char *dbdir);
 int filecopy(const char *src, const char *dest);
 int isnumb(const char *str);
 int dircopy(const char *src, const char *dest);
 int cvd_unpack(const char *cvd, const char *destdir);
-void daemonize(void);
+int daemonize(void);
 
 #endif
